import { StartLoadingMetadata } from './../store/actions';
import { Store } from '@ngrx/store';
import { GlobalState } from '@app/core/state';
import { WebsocketService } from './../../shared/services/websocket.service';
import {
  Component,
  Input,
  Output,
  EventEmitter,
  ViewChild,
  OnInit,
  OnDestroy,
} from '@angular/core';
import { MatSnackBar } from '@angular/material/snack-bar';
import { AuditMetadata } from '../models/audit-metadata';
import { MatPaginator } from '@angular/material/paginator';
import { MatTableDataSource } from '@angular/material/table';
import { Observable, Subscription } from 'rxjs';

export class AuditStatusMessage {
  status: string;
  jobId: string;
  netId: string;
  type: string;
}

@Component({
  selector: 'cse-dars-metadata-table',
  templateUrl: './metadata-table.component.html',
  styleUrls: ['./metadata-table.component.scss'],
})
export class DarsMetadataTableComponent implements OnInit, OnDestroy {
  @Input() public metadata: Observable<AuditMetadata[]>;
  @Input() public title: string;
  @Input() public tagline: string;
  @Input() public button: string;

  @Output() public openAudit = new EventEmitter<AuditMetadata>();
  @Output() buttonClick = new EventEmitter();

  @ViewChild(MatPaginator) paginator: MatPaginator;

  public dataSource: MatTableDataSource<AuditMetadata>;
  public messageSub: Subscription;
  public newMessage: AuditStatusMessage;
  public displayedColumns = [
    'view',
    'school',
    'program',
    'honors',
    'plan',
    'status',
    'download',
  ];

  constructor(
    private websocketService: WebsocketService,
    private store: Store<GlobalState>,
    private snackBar: MatSnackBar,
  ) {
    this.messageSub = websocketService.messages.subscribe(message => {
      this.newMessage = JSON.parse(message);
      if (this.newMessage && this.newMessage.type === 'auditStatus') {
        this.store.dispatch(new StartLoadingMetadata());
        this.snackBar.open('Audit Complete');
      }
    });
  }

  public static sortMetadata(metadata: AuditMetadata[]): AuditMetadata[] {
    return metadata.sort((a, b) => {
      const aDate = new Date(a.darsAuditRunDate);
      const bDate = new Date(b.darsAuditRunDate);

      if (aDate < bDate) {
        return 1;
      }
      if (aDate > bDate) {
        return -1;
      }
      return 0;
    });
  }

  public ngOnInit() {
    this.metadata.subscribe(metadata => {
      const sorted = DarsMetadataTableComponent.sortMetadata(metadata);
      this.dataSource = new MatTableDataSource<AuditMetadata>(sorted);
      this.dataSource.paginator = this.paginator;
    });
  }

  public ngOnDestroy() {
    this.messageSub.unsubscribe();
  }
}
